@@ -16,7 +16,7 @@ use Rose::DB::Constants qw(IN_TRANSACTION);
 use Rose::DB::Object::Exception;
 use Rose::DB::Object::Util();
 
-our $VERSION = '0.784';
+our $VERSION = '0.815';
 
 our $Debug = 0;
 
@@ -112,7 +112,15 @@ sub _init_db
     return $db;
   }
 
-  $self->error("Could not init_db() - $error - " . ($db ? $db->error : ''));
+  if(ref $error)
+  {
+    $self->error($error);
+  }
+  else
+  {
+    $self->error("Could not init_db() - $error - " . ($db ? $db->error : ''));
+  }
+
   $self->meta->handle_error($self);
   return undef;
 }
@@ -154,6 +162,7 @@ sub load
     $meta->dbi_prepare_cached;
 
   local $self->{STATE_SAVING()} = 1;
+  local $self->{SAVING_FOR_LOAD()} = 1;
 
   my(@key_columns, @key_methods, @key_values);
 
@@ -259,6 +268,12 @@ sub load
     $column_names = $meta->column_names;
   }
 
+  # Coerce for_update boolean alias into lock argument
+  if(delete $args{'for_update'})
+  {
+    $args{'lock'}{'type'} ||= 'for update';
+  }
+
   #
   # Handle sub-object load in separate code path
   #
@@ -286,6 +301,7 @@ sub load
                                   multi_many_ok  => 1,
                                   nonlazy        => $args{'nonlazy'},
                                   inject_results => $args{'inject_results'},
+                                  lock           => $args{'lock'},
                                   (exists $args{'prepare_cached'} ?
                                   (prepare_cached =>  $args{'prepare_cached'}) : 
                                   ()))
@@ -302,7 +318,7 @@ sub load
 
     if($error)
     {
-      $self->error("load(with => ...) - $error");
+      $self->error(ref $error ? $error : "load(with => ...) - $error");
       $meta->handle_error($self);
       return undef;
     }
@@ -393,6 +409,11 @@ sub load
         }
       }
 
+      if(my $lock = $args{'lock'})
+      {
+        $sql .= ' ' . $db->format_select_lock($self, $lock);
+      }
+
       # $meta->prepare_select_options (defunct)
       $sth = $prepare_cached ? $dbh->prepare_cached($sql, undef, 3) : 
                                $dbh->prepare($sql);
@@ -451,7 +472,7 @@ sub load
 
   if($error)
   {
-    $self->error("load() - $error");
+    $self->error(ref $error ? $error : "load() - $error");
     $meta->handle_error($self);
     return undef;
   }
@@ -496,7 +517,9 @@ sub save
 
     unless($ret)
     {
-      $self->error('Could not begin transaction before saving - ' . $db->error);
+      my $error = $db->error;
+      $self->error(ref $error ? $error : "Could not begin transaction before saving - $error");
+      $self->meta->handle_error($self);
       return undef;
     }
 
@@ -715,7 +738,8 @@ sub update
   #
   #unless($ret)
   #{
-  #  $self->error('Could not begin transaction before updating - ' . $db->error);
+  #  my $error = $db->error;
+  #  $self->error(ref $error ? $error : "Could not begin transaction before updating - $error");
   #  return undef;
   #}
   #
@@ -941,7 +965,7 @@ sub update
 
   if($error)
   {
-    $self->error("update() - $error");
+    $self->error(ref $error ? $error : "update() - $error");
     #$db->rollback or warn $db->error  if($started_new_tx);
     $self->meta->handle_error($self);
     return 0;
@@ -979,7 +1003,8 @@ sub insert
   #
   #unless($ret)
   #{
-  #  $self->error('Could not begin transaction before inserting - ' . $db->error);
+  #  my $error = $db->error;
+  #  $self->error(ref $error ? $error : "Could not begin transaction before inserting - $error");
   #  return undef;
   #}
   #
@@ -1231,7 +1256,7 @@ sub insert
 
   if($error)
   {
-    $self->error("insert() - $error");
+    $self->error(ref $error ? $error : "insert() - $error");
     #$db->rollback or warn $db->error  if($started_new_tx);
     $self->meta->handle_error($self);
     return 0;
@@ -1322,8 +1347,10 @@ sub delete
             my $column_map = $relationship->column_map;
             my @query;
 
-            while(my($local_column, $foreign_column) = each(%$column_map))
+            foreach my $local_column (keys %$column_map)
             {
+              my $foreign_column = $column_map->{$local_column};
+
               my $method = $meta->column_accessor_method_name($local_column);
               my $value =  $self->$method();
 
@@ -1367,8 +1394,10 @@ sub delete
             my @query;
 
             # "Local" here means "local to the mapping table"
-            while(my($local_column, $foreign_column) = each(%$key_columns))
+            foreach my $local_column (keys %$key_columns)
             {
+              my $foreign_column = $key_columns->{$local_column};
+
               my $method = $meta->column_accessor_method_name($foreign_column);
               my $value  = $self->$method();
 
@@ -1429,8 +1458,10 @@ sub delete
           my $key_columns = $fk->key_columns;
           my @query;
 
-          while(my($local_column, $foreign_column) = each(%$key_columns))
+          foreach my $local_column (keys %$key_columns)
           {
+            my $foreign_column = $key_columns->{$local_column};
+
             my $method = $meta->column_accessor_method_name($local_column);
             my $value =  $self->$method();
 
@@ -1466,8 +1497,10 @@ sub delete
           my $column_map = $relationship->column_map;
           my @query;
 
-          while(my($local_column, $foreign_column) = each(%$column_map))
+          foreach my $local_column (keys %$column_map)
           {
+            my $foreign_column = $column_map->{$local_column};
+
             my $method = $meta->column_accessor_method_name($local_column);
             my $value =  $self->$method();
 
@@ -1509,7 +1542,7 @@ sub delete
     if($error)
     {
       Rose::DB::Object::Manager->error_mode($mgr_error_mode);
-      $self->error("delete() with cascade - $error");
+      $self->error(ref $error ? $error : "delete() with cascade - $error");
       $db->rollback  if($db && $started_new_tx);
       $self->meta->handle_error($self);
       return 0;
@@ -1554,7 +1587,7 @@ sub delete
 
     if($error)
     {
-      $self->error("delete() - $error");
+      $self->error(ref $error ? $error : "delete() - $error");
       $self->meta->handle_error($self);
       return 0;
     }
@@ -1623,7 +1656,7 @@ EOF
   {
     Carp::confess qq(Can't locate $method_type method "$2" via package "$1"$msg);
   }
-  else # notreached?
+  else # not reached?
   {
     Carp::confess qq(Can't locate $method_type method $AUTOLOAD$msg);
   }
@@ -1913,7 +1946,7 @@ Finally, the L<Rose::DB::Object::Loader> can be used to automatically create a s
 
 Before L<Rose::DB::Object> can do any useful work, you must register at least one L<Rose::DB> data source.  By default, L<Rose::DB::Object> instantiates a L<Rose::DB> object by passing no arguments to its constructor.  (See the L<db|/db> method.)  If you register a L<Rose::DB> data source using the default type and domain, this will work fine.  Otherwise, you must override the L<meta|/meta> method in your L<Rose::DB::Object> subclass and have it return the appropriate L<Rose::DB>-derived object.
 
-To define your own L<Rose::DB::Object>-derived class, you must describe the table that your class will act as a front-end for.    This is done through the L<Rose::DB::Object::Metadata> object associated with each L<Rose::DB::Object>-dervied class.  The metadata object is accessible via L<Rose::DB::Object>'s L<meta|/meta> method.
+To define your own L<Rose::DB::Object>-derived class, you must describe the table that your class will act as a front-end for.    This is done through the L<Rose::DB::Object::Metadata> object associated with each L<Rose::DB::Object>-derived class.  The metadata object is accessible via L<Rose::DB::Object>'s L<meta|/meta> method.
 
 Metadata objects can be populated manually or automatically.  Both techniques are shown in the L<synopsis|/SYNOPSIS> above.  The automatic mode works by asking the database itself for the information.  There are some caveats to this approach.  See the L<auto-initialization|Rose::DB::Object::Metadata/"AUTO-INITIALIZATION"> section of the L<Rose::DB::Object::Metadata> documentation for more information.
 
@@ -1997,11 +2030,12 @@ Since Oracle does not natively support a serial or auto-incremented column data
     CREATE TRIGGER mytable_insert BEFORE INSERT ON mytable
     FOR EACH ROW
     BEGIN
-        SELECT NVL(:new.id, mytable_id_seq.nextval) 
-          INTO :new.id FROM dual;
+       IF :new.id IS NULL THEN
+           :new.id := mytable_id_seq.nextval;   
+       END IF;
     END;
 
-Note the use of C<NVL()> in the trigger, which allows the value of the C<id> column to be set explicitly.  If a non-NULL value for the C<id> column is provided, then a new value is not pulled from the sequence.
+Note the conditional that checks if C<:new.id> is null, which allows the value of the C<id> column to be set explicitly.  If a non-NULL value for the C<id> column is provided, then a new value is not pulled from the sequence.
 
 If the sequence is not named according to the C<E<lt>tableE<gt>_E<lt>columnE<gt>_seq> convention, you can specify the sequence name explicitly in the column metadata.  Example:
 
@@ -2194,7 +2228,7 @@ If true, then L<DBI>'s L<prepare_cached|DBI/prepare_cached> method will be used
 
 =back
 
-The cascaded delete feature described above plays it safe by only deleting rows that are not referenced by any other rows (according to the metadata provided by each L<Rose::DB::Object>-derived class).  I B<strongly recommend> that you implement "cascaded delete" in the database itself, rather than using this feature.  It will undoubtedly be faster and more robust than doing it "client-side."  You may also want to cascade only to certain tables, or otherwise deviate from the "safe" plan.  If your database supports automatic cascaded delete and/or triggers, please consider using thse features.
+The cascaded delete feature described above plays it safe by only deleting rows that are not referenced by any other rows (according to the metadata provided by each L<Rose::DB::Object>-derived class).  I B<strongly recommend> that you implement "cascaded delete" in the database itself, rather than using this feature.  It will undoubtedly be faster and more robust than doing it "client-side."  You may also want to cascade only to certain tables, or otherwise deviate from the "safe" plan.  If your database supports automatic cascaded delete and/or triggers, please consider using these features.
 
 =item B<error>
 
@@ -2234,6 +2268,44 @@ PARAMS are optional name/value pairs.  Valid PARAMS are:
 
 =over 4
 
+=item B<for_update BOOL>
+
+If true, this parameter is translated to be the equivalent of passing the L<lock|/lock> parameter and setting the C<type> to C<for update>.  For example, these are both equivalent:
+
+    $object->load(for_update => 1);
+    $object->load(lock => { type => 'for update' });
+
+See the L<lock|/lock> parameter below for more information.
+
+=item B<lock [ TYPE | HASHREF ]>
+
+Load the object using some form of locking.  These lock directives have database-specific behavior and not all directives are supported by all databases.  The value should be a reference to a hash or a TYPE string, which is equivalent to setting the value of the C<type> key in the hash reference form.  For example, these are both equivalent:
+
+    $object->load(lock => 'for update');
+    $object->load(lock => { type => 'for update' });
+
+Valid hash keys are:
+
+=over 4
+
+=item B<columns ARRAYREF>
+
+A reference to an array of column names to lock.  References to scalars will be de-referenced and used as-is, included literally in the SQL locking clause.
+
+=item C<nowait BOOL>
+
+If true, do not wait to acquire the lock.    If supported, this is usually by adding a C<NOWAIT> directive to the SQL.
+
+=item C<type TYPE>
+
+The lock type.  Valid values for TYPE are C<for update> and C<shared>.  This parameter is required unless the L<for_update|/for_update> parameter was passed with a true value.
+
+=item C<wait TIME>
+
+Wait for the specified TIME (generally seconds) before giving up acquiring the lock.  If supported, this is usually by adding a C<WAIT ...> clause to the SQL.
+
+=back
+
 =item B<nonlazy BOOL>
 
 If true, then all columns will be fetched from the database, even L<lazy|Rose::DB::Object::Metadata::Column/load_on_demand> columns.  If omitted, the default is false.
@@ -2424,11 +2496,11 @@ L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Rose-DB-Object>
 
 There's also a wiki and other resources linked from the Rose project home page:
 
-L<http://rose.googlecode.com>
+L<http://rosecode.org>
 
 =head1 CONTRIBUTORS
 
-Bradley C Bailey, Graham Barr, David Christensen, Lucian Dragus, Justin Ellison, Perrin Harkins, Cees Hek, Benjamin Hitz, Peter Karman, Ed Loehr, Michael Reece, Thomas Whaples, Teodor Zlatanov
+Bradley C Bailey, Graham Barr, Kostas Chatzikokolakis, David Christensen, Lucian Dragus, Justin Ellison, Perrin Harkins, Cees Hek, Benjamin Hitz, Dave Howorth, Peter Karman, Ed Loehr, Adam Mackler, Michael Reece, Thomas Whaples, Douglas Wilson, Teodor Zlatanov
 
 =head1 AUTHOR
 
@@ -2436,6 +2508,6 @@ John C. Siracusa (siracusa@gmail.com)
 
 =head1 LICENSE
 
-Copyright (c) 2009 by John C. Siracusa.  All rights reserved.  This program is
+Copyright (c) 2010 by John C. Siracusa.  All rights reserved.  This program is
 free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
