@@ -7,7 +7,7 @@ use Rose::DB::Object::MakeMethods::Date;
 use Rose::DB::Object::Metadata::Column::Date;
 our @ISA = qw(Rose::DB::Object::Metadata::Column::Date);
 
-our $VERSION = '0.70';
+our $VERSION = '0.788';
 
 __PACKAGE__->add_common_method_maker_argument_names
 (
@@ -29,10 +29,10 @@ sub type { 'datetime' }
 
 sub should_inline_value
 {
-  #my($self, $db, $value) = @_;
-  my $driver = $_[1]->driver;
-  return ($_[1]->validate_datetime_keyword($_[2]) && 
-          (($driver eq 'informix' || $driver eq 'sqlite') || $_[2] =~ /^\w+\(.*\)$/)) ? 1 : 0;
+  my($self, $db, $value) = @_;
+  no warnings 'uninitialized';
+  return (($db->validate_datetime_keyword($value) && $db->should_inline_datetime_keyword($value)) ||
+          ($db->keyword_function_calls && $value =~ /^\w+\(.*\)$/)) ? 1 : 0;
 }
 
 sub parse_value
@@ -128,8 +128,8 @@ Returns "datetime".
 
 John C. Siracusa (siracusa@gmail.com)
 
-=head1 COPYRIGHT
+=head1 LICENSE
 
-Copyright (c) 2007 by John C. Siracusa.  All rights reserved.  This program is
+Copyright (c) 2010 by John C. Siracusa.  All rights reserved.  This program is
 free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
